<?php

namespace CreoCode\ContaoTourismBundle\Modules;

use Contao\CoreBundle\Exception\InternalServerErrorException;
use Contao\CoreBundle\Exception\PageNotFoundException;
use Patchwork\Utf8;

use Contao\Config;
use Contao\Input;
use Contao\StringUtil;
use Contao\System;
use Contao\Environment;
use Contao\BackendTemplate;

use CreoCode\ContaoTourismBundle\Models\AccommodationsModel;

/**
 * Front end module "accommodations reader".
 *
 * @property array    $accommodations_sets
 */
class ModuleAccommodationsReader extends ModuleAccommodations
{

	/**
	 * Template
	 * @var string
	 */
	protected $strTemplate = 'mod_accommodationsreader';

	/**
	 * Display a wildcard in the back end
	 *
	 * @throws InternalServerErrorException
	 *
	 * @return string
	 */
	public function generate()
	{
		if (TL_MODE == 'BE')
		{
			$objTemplate = new BackendTemplate('be_wildcard');
			$objTemplate->wildcard = '### ' . Utf8::strtoupper($GLOBALS['TL_LANG']['FMD']['accommodationsreader'][0]) . ' ###';
			$objTemplate->title = $this->headline;
			$objTemplate->id = $this->id;
			$objTemplate->link = $this->name;
			$objTemplate->href = 'contao/main.php?do=themes&amp;table=tl_module&amp;act=edit&amp;id=' . $this->id;

			return $objTemplate->parse();
		}

		// Set the item from the auto_item parameter
		if (!isset($_GET['items']) && Config::get('useAutoItem') && isset($_GET['auto_item']))
		{
			Input::setGet('items', Input::get('auto_item'));
		}

		// Return an empty string if "items" is not set (to combine list and reader on same page)
		if (!Input::get('items'))
		{
			return '';
		}

		$this->accommodations_sets = $this->sortOutProtected(StringUtil::deserialize($this->accommodations_sets));

		if (empty($this->accommodations_sets) || !\is_array($this->accommodations_sets))
		{
			throw new InternalServerErrorException('The accommodations reader ID ' . $this->id . ' has no sets specified.', $this->id);
		}

		return parent::generate();
	}

	/**
	 * Generate the module
	 */
	protected function compile()
	{
		/** @var PageModel $objPage */
		global $objPage;

		$this->Template->articles = '';
		$this->Template->referer = 'javascript:history.go(-1)';
		$this->Template->back = $GLOBALS['TL_LANG']['MSC']['goBack'];

		// Get the accommodations item
		$objArticle = AccommodationsModel::findPublishedByParentAndIdOrAlias(Input::get('items'), $this->accommodations_sets);

		// The accommodations item does not exist or has an external target (see #33)
		if (null === $objArticle || $objArticle->source != 'default')
		{
			throw new PageNotFoundException('Page not found: ' . Environment::get('uri'));
		}

		$arrArticle = $this->parseArticle($objArticle);
		$this->Template->articles = $arrArticle;

		// Overwrite the page title (see #2853, #4955 and #87)
		if ($objArticle->pageTitle)
		{
			$objPage->pageTitle = $objArticle->pageTitle;
		}
		elseif ($objArticle->headline)
		{
			$objPage->pageTitle = strip_tags(StringUtil::stripInsertTags($objArticle->headline));
		}

		// Overwrite the page description
		if ($objArticle->description)
		{
			$objPage->description = $objArticle->description;
		}
		elseif ($objArticle->teaser)
		{
			$objPage->description = $this->prepareMetaDescription($objArticle->teaser);
		}
	}
}

class_alias(ModuleAccommodationsReader::class, 'ModuleAccommodationsReader');
